import traceback
from typing import Optional

from cpl_core.configuration.configuration_model_abc import ConfigurationModelABC
from cpl_core.console import Console
from cpl_query.extension import List

from gismo_core.model.embed_description import EmbedDescription
from gismo_core.model.embed_description_field import EmbedDescriptionField


class EmbedDescriptionSettings(ConfigurationModelABC):

    def __init__(self):
        ConfigurationModelABC.__init__(self)

        self._embed_description: Optional[EmbedDescription] = None

    @property
    def embed_description(self) -> EmbedDescription:
        return self._embed_description

    def from_dict(self, settings: dict):
        try:
            title = ''
            description = ''
            url = ''
            color = ''
            fields = List(EmbedDescriptionField)
            footer = ''

            for key in settings:
                if key == 'Title':
                    title = settings[key]
                elif key == 'Description':
                    description = settings[key]
                elif key == 'URL':
                    url = settings[key]
                elif key == 'Color':
                    color = settings[key]
                elif key == 'Fields':
                    for field in settings[key]:
                        fields.append(EmbedDescriptionField(field['Name'], field['Value'], bool(field['Inline'])))
                elif key == 'Footer':
                    footer = settings[key]
                else:
                    raise Exception(f'Unexpected key {key}')

            self._embed_description = EmbedDescription(title, description, url, color, fields, footer)

        except Exception as e:
            Console.error(f'[ ERROR ] [ {__name__} ]: Reading error in {self.__name__} settings')
            Console.error(f'[ EXCEPTION ] [ {__name__} ]: {e} -> {traceback.format_exc()}')
