from datetime import datetime

from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_query.extension import List
from discord.ext import commands
from discord.ext.commands import Context

import gismo
from gismo_core.abc.bot_service_abc import BotServiceABC
from gismo_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from gismo_core.abc.command_abc import CommandABC
from gismo_core.abc.message_service_abc import MessageServiceABC
from gismo_core.abc.module_abc import ModuleABC
from gismo_core.model.embed_description_field import EmbedDescriptionField
from gismo_core.service.embed_service import EmbedService
from modules.base.base_settings import BaseSettings


class InfoCommandService(CommandABC):

    def __init__(
            self,
            config: ConfigurationABC,
            logger: LoggerABC,
            message_service: MessageServiceABC,
            bot: BotServiceABC,
            client_utils: ClientUtilsServiceABC
    ):
        CommandABC.__init__(self)

        self._config = config
        self._logger = logger
        self._message_service = message_service
        self._bot = bot
        self._client_utils = client_utils

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.command()
    async def info(self, ctx: Context):
        self._logger.debug(__name__, f'Received command info {ctx}')
        self._client_utils.received_command(ctx.guild.id)
        settings: BaseSettings = self._config.get_configuration(f'Base_{ctx.guild.id}')
        embed_description = settings.info_command_message.embed_description
        client = self._client_utils.get_client(self._bot.user.id, ctx.guild.id)

        for i in range(len(embed_description.fields)):
            field = embed_description.fields[i]
            if field.value == '$version':
                field.value = gismo.__version__
            elif field.value == '$ontime':
                start_time = self._config.get_configuration('Bot_StartTime')
                ontime = round((datetime.now() - datetime.strptime(start_time, '%Y-%m-%d %H:%M:%S.%f')).total_seconds()/3600, 2)
                field.value = f'{ontime}h'
            elif field.value == '$sent_message_count':
                field.value = client.sent_message_count
            elif field.value == '$received_message_count':
                field.value = client.received_message_count
            elif field.value == '$deleted_message_count':
                field.value = client.deleted_message_count
            elif field.value == '$received_command_count':
                field.value = client.received_command_count
            elif field.value == '$moved_users_count':
                field.value = client.moved_users_count
            elif field.value == '$modules':
                field.value = ''
                for module in ModuleABC.__subclasses__():
                    field.value += f'{module.__name__}\n'

        await self._message_service.send_ctx_msg(
            ctx,
            EmbedService.get_embed(embed_description)
        )
        self._logger.trace(__name__, f'Finished ping command')
